// a lib to help you invoking PGOS backend Http API
package pgos_lib

import (
	"bytes"
	"crypto/aes"
	"crypto/cipher"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net"
	"net/http"
	"strings"
	"time"
)

const ServerTokenIv string = "$3,.'/&^rgnjkl!#"

type AesCbc struct {
	key []byte
	iv  []byte
}

func NewAesCbc(key, iv []byte) *AesCbc {
	return &AesCbc{key: key, iv: iv}
}

func (ac *AesCbc) Encrypt(text string) (string, error) {
	plainText := []byte(text)
	paddedPlainText := Pad(plainText)
	block, err := aes.NewCipher(ac.key)
	if err != nil {
		return "", err
	}
	ciphertext := make([]byte, len(paddedPlainText))
	mode := cipher.NewCBCEncrypter(block, ac.iv)
	mode.CryptBlocks(ciphertext, paddedPlainText)
	return base64.StdEncoding.EncodeToString(ciphertext), nil
}

func (ac *AesCbc) Decrypt(text string) (string, error) {
	ciphertext, err := base64.StdEncoding.DecodeString(text)
	if err != nil {
		return "", err
	}
	block, err := aes.NewCipher(ac.key)
	if err != nil {
		return "", err
	}
	plaintext := make([]byte, len(ciphertext))
	mode := cipher.NewCBCDecrypter(block, ac.iv)
	mode.CryptBlocks(plaintext, ciphertext)
	unpaddedPlaintext := UnPad(plaintext)
	return string(unpaddedPlaintext), nil
}

func Pad(text []byte) []byte {
	padding := aes.BlockSize - (len(text) % aes.BlockSize)
	padText := bytes.Repeat([]byte{byte(padding)}, padding)
	return append(text, padText...)
}

func UnPad(text []byte) []byte {
	length := len(text)
	unPadding := int(text[length-1])
	return text[:(length - unPadding)]
}

func EncryptServerTicket(key, ticketJsonStr string) (string, error) {
	ac := NewAesCbc([]byte(key), []byte(ServerTokenIv))
	return ac.Encrypt(ticketJsonStr)
}

func DecryptServerTicket(key, ticketJsonStr string) (string, error) {
	ac := NewAesCbc([]byte(key), []byte(ServerTokenIv))
	return ac.Decrypt(ticketJsonStr)
}

type SecretTicket struct {
	TitleRegionID string `json:"title_region_id"`
	SecretID      string `json:"secret_id"`
	Time          int64  `json:"time"`
}

func GenServerTicket(secretID, secretKey, titleRegionID string) (string, error) {
	st := SecretTicket{
		TitleRegionID: titleRegionID,
		SecretID:      secretID,
		Time:          time.Now().Unix(),
	}
	ticketJson, err := json.Marshal(st)
	if err != nil {
		return "", err
	}
	key := strings.Join(strings.Split(secretKey, "-"), "")
	ticket, err := EncryptServerTicket(key, string(ticketJson))
	return ticket, err
}

func SplitServerKey(serverKey string) (string, string) {
	s := strings.Split(serverKey, "-")
	if len(s) > 1 {
		secretID := s[0]
		secretKey := strings.Join(s[1:], "-")
		return secretID, secretKey
	}
	return "", ""
}

func GetServerUrl(titleRegionID string) string {
	location := "euff"
	domainSuffix := "com"
	arr := strings.Split(titleRegionID, "_")
	if len(arr) > 0 {
		location = arr[0]
	}
	if location == "t" || location == "d" || strings.HasPrefix(location, "cn") {
		domainSuffix = "cn"
	}
	serverUrl := fmt.Sprintf("https://%s.server.pgos.intlgame.%s", location, domainSuffix)
	return serverUrl
}

func InvokeBackendApi(titleID, titleRegionID, serverKey, apiUri string, body []byte) (*http.Header, []byte, error) {
	secretID, secretKey := SplitServerKey(serverKey)
	ticket, _ := GenServerTicket(secretID, secretKey, titleRegionID)
	url := GetServerUrl(titleRegionID) + apiUri
	defaultTransport := &http.Transport{}
	defaultTransport.DialContext = (&net.Dialer{}).DialContext
	client := &http.Client{
		Timeout:   time.Duration(3) * time.Second,
		Transport: defaultTransport,
	}
	req, err := http.NewRequest("POST", url, bytes.NewBuffer(body))
	if err != nil {
		return nil, nil, err
	}
	req.Header.Set("Content-Type", "application/json")
	req.Header.Set("TitleId", titleID)
	req.Header.Set("TitleRegionId", titleRegionID)
	req.Header.Set("SecretId", secretID)
	req.Header.Set("ServerTicket", ticket)

	//fmt.Printf("Request %s:\n", url)
	//fmt.Printf("Headers:\n")
	//for k := range req.Header {
	//fmt.Printf("%s: %s\n", k, req.Header.Get(k))
	//}
	//fmt.Printf("Body:\n")
	//fmt.Println(bodyStr)
	//fmt.Printf("---------------------------------\n")

	resp, err := client.Do(req)
	if err != nil {
		return nil, nil, err
	}
	defer resp.Body.Close()

	rspBody, err := ioutil.ReadAll(resp.Body)
	/*fmt.Printf("Response %s\n", resp.Status)
	fmt.Printf("Headers:\n")
	for k := range resp.Header {
		fmt.Printf("%s: %s\n", k, resp.Header.Get(k))
	}*/

	return &resp.Header, rspBody, nil
}
